'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var language = require('@codemirror/language');
var autocomplete = require('@codemirror/autocomplete');
var lr = require('@lezer/lr');
var highlight = require('@lezer/highlight');
var lint = require('@codemirror/lint');

var collection = {
	shortDescription: "The constraint specifies which entity collection will be searched for results.",
	userDocsLink: "https://evitadb.io/documentation/query/basics#header"
};
var filterGroupBy = {
	shortDescription: "The container encapsulating filter constraint limiting the facet groups returned in facet summary.",
	userDocsLink: "https://evitadb.io/documentation/query/basics#filter-by"
};
var attributeGreaterThanEquals = {
	shortDescription: "Compares value of the attribute with passed value and checks if the value of that attribute is greater than or equals to the passed value.",
	userDocsLink: "https://evitadb.io/documentation/query/filtering/comparable#attribute-greater-than-equals"
};
var filterBy = {
	shortDescription: "The container encapsulating inner filter constraint into one main constraint that is required by the query.",
	userDocsLink: "https://evitadb.io/documentation/query/basics#filter-by"
};
var priceBetween = {
	shortDescription: "The constraint checks if entity has price for sale within the passed range of prices (both ends are inclusive).",
	userDocsLink: "https://evitadb.io/documentation/query/filtering/price#price-between"
};
var entityHaving = {
	shortDescription: "The container allowing to filter entities by having references to entities managed by evitaDB that  + match inner filtering constraints. This container resembles the SQL inner join clauses where the `entityHaving` + contains the filtering condition on particular join.",
	userDocsLink: "https://evitadb.io/documentation/query/filtering/references#entity-having"
};
var priceInCurrency = {
	shortDescription: "The constraint filters out all entities that lack selling price in specified currency.",
	userDocsLink: "https://evitadb.io/documentation/query/filtering/price#price-in-currency"
};
var userFilter = {
	shortDescription: "The container for constraints that are controlled by the user (client UI widgets).  + It is used mainly to distinguish between user constraint (refining the search) and program defined  + constraints (considered mandatory), when the extra results are computed.",
	userDocsLink: "https://evitadb.io/documentation/query/filtering/behavioral#user-filter"
};
var attributeContains = {
	shortDescription: "Compares value of the attribute with passed value and checks if the text value of that attribute contains part of passed text (case-sensitive).",
	userDocsLink: "https://evitadb.io/documentation/query/filtering/string#attribute-contains"
};
var excluding = {
	shortDescription: "The constraint narrows hierarchy within parent constraint to exclude specified hierarchy subtrees from search.",
	userDocsLink: "https://evitadb.io/documentation/query/filtering/hierarchy#excluding"
};
var priceValidIn = {
	shortDescription: "The constraint checks if entity has selling price valid at the passed moment.",
	userDocsLink: "https://evitadb.io/documentation/filtering/price#price-valid-in"
};
var attributeLessThan = {
	shortDescription: "Compares value of the attribute with passed value and checks if the value of that attribute is less than the passed value.",
	userDocsLink: "https://evitadb.io/documentation/query/filtering/comparable#attribute-less-than"
};
var priceInPriceLists = {
	shortDescription: "The constraint filters out all entities that lack selling price in specified price lists.  + Order of price lists also defines priority for selecting the entity selling price - the price from first price  + list in the list will be used as a selling price for the entire entity.",
	userDocsLink: "https://evitadb.io/documentation/query/filtering/price#price-in-price-lists"
};
var entityLocaleEquals = {
	shortDescription: "The constraint if at least one of entity locales (derived from entity attributes or associated data) equals to the passed one.",
	userDocsLink: "https://evitadb.io/documentation/query/filtering/locale#entity-locale-equals"
};
var attributeGreaterThan = {
	shortDescription: "Compares value of the attribute with passed value and checks if the value of that attribute is greater than the passed value.",
	userDocsLink: "https://evitadb.io/documentation/query/filtering/comparable#attribute-greater-than"
};
var facetHaving = {
	shortDescription: "The container allowing to filter entities by having references to entities managed by evitaDB that  + match the inner filter constraint. This container resembles the SQL inner join clauses and works in cooperation  + with facet summary requirement.",
	userDocsLink: "https://evitadb.io/documentation/query/filtering/references#facet-having"
};
var or = {
	shortDescription: "The container that combines inner constraints with [logical OR](https://en.wikipedia.org/wiki/Logical_disjunction).",
	userDocsLink: "https://evitadb.io/documentation/query/filtering/logical#or"
};
var attributeEquals = {
	shortDescription: "Compares value of the attribute with passed value and checks if they are both equal.",
	userDocsLink: "https://evitadb.io/documentation/query/filtering/comparable#attribute-equals"
};
var hierarchyWithinRoot = {
	shortDescription: "The constraint if entity is placed inside the defined hierarchy tree starting at the root of the tree (or has reference to any hierarchical entity in the tree).",
	userDocsLink: "https://evitadb.io/documentation/query/filtering/hierarchy#hierarchy-within-root"
};
var and = {
	shortDescription: "The container that combines inner constraints with [logical AND](https://en.wikipedia.org/wiki/Logical_conjunction).",
	userDocsLink: "https://evitadb.io/documentation/query/filtering/logical#and"
};
var attributeBetween = {
	shortDescription: "Compares value of the attribute with passed value and checks if the value of that attribute is within the passed range (both ends are inclusive).",
	userDocsLink: "https://evitadb.io/documentation/query/filtering/comparable#attribute-between"
};
var referenceHaving = {
	shortDescription: "The container allowing to filter entities by having references to entities managed by evitaDB that  + match the inner filter constraint. This container resembles the SQL inner join clauses.",
	userDocsLink: "https://evitadb.io/documentation/query/filtering/references#reference-having"
};
var not = {
	shortDescription: "The container that behaves as [logical NOT](https://en.wikipedia.org/wiki/Negation) for the inner constraint.",
	userDocsLink: "https://evitadb.io/documentation/query/filtering/logical#not"
};
var attributeIs = {
	shortDescription: "The constraint if value of the attribute is same as passed special value.",
	userDocsLink: "https://evitadb.io/documentation/query/filtering/comparable#attribute-is"
};
var attributeLessThanEquals = {
	shortDescription: "Compares value of the attribute with passed value and checks if the value of that attribute is less than or equals to the passed value.",
	userDocsLink: "https://evitadb.io/documentation/query/filtering/comparable#attribute-less-than-equals"
};
var having = {
	shortDescription: "The constraint narrows hierarchy within parent constraint to include specified hierarchy subtrees from search.",
	userDocsLink: "https://evitadb.io/documentation/query/filtering/hierarchy#having"
};
var attributeStartsWith = {
	shortDescription: "Compares value of the attribute with passed value and checks if the text value of that attributes starts with passed text. (case-sensitive)",
	userDocsLink: "https://evitadb.io/documentation/query/filtering/string#attribute-starts-with"
};
var hierarchyWithin = {
	shortDescription: "The constraint if entity is placed inside the defined hierarchy tree (or has reference to any hierarchical entity in the tree).",
	userDocsLink: "https://evitadb.io/documentation/query/filtering/hierarchy#hierarchy-within"
};
var directRelation = {
	shortDescription: "The constraint limits hierarchy within parent constraint to take only directly related entities into an account.",
	userDocsLink: "https://evitadb.io/documentation/query/filtering/hierarchy#direct-relation"
};
var attributeEndsWith = {
	shortDescription: "Compares value of the attribute with passed value and checks if the text value of that attribute ends with passed text (case-sensitive).",
	userDocsLink: "https://evitadb.io/documentation/query/filtering/string#attribute-ends-with"
};
var excludingRoot = {
	shortDescription: "The constraint limits hierarchy within parent constraint to exclude the entities directly related to the searched root node.",
	userDocsLink: "https://evitadb.io/documentation/query/filtering/hierarchy#excluding-root"
};
var attributeInSet = {
	shortDescription: "Compares value of the attribute with passed value and checks if the value of that attribute  + equals to at least one of the passed values.  + The constraint is equivalent to the multiple `equals` constraints combined with logical OR.",
	userDocsLink: "https://evitadb.io/documentation/query/filtering/comparable#attribute-in-set"
};
var entityPrimaryKeyInSet = {
	shortDescription: "The constraint checks if primary key of the entity equals to at least one of the passed values.  + The constraint is equivalent to one or more `equals` constraints combined with logical OR.",
	userDocsLink: "https://evitadb.io/documentation/query/filtering/constant#entity-primary-key-in-set"
};
var attributeInRange = {
	shortDescription: "Compares value of the attribute with passed value and checks if the range value of that  + attribute contains the passed value within its limits (both ends are inclusive).  + The constraint can be used only for Range data type values.",
	userDocsLink: "https://evitadb.io/documentation/query/filtering/range#attribute-in-range"
};
var entityProperty = {
	shortDescription: "The constraint sorts returned references by applying ordering constraint on referenced entity.",
	userDocsLink: "https://evitadb.io/documentation/query/ordering/reference#entity-property"
};
var entityPrimaryKeyInFilter = {
	shortDescription: "The constraint sorts returned entities by ordering of the values specified `entityPrimaryKeysInSet` in filter.",
	userDocsLink: "https://evitadb.io/documentation/query/ordering/constant#exact-entity-primary-key-order-used-in-filter"
};
var referenceProperty = {
	shortDescription: "The constraint sorts returned entities or references by attribute specified on its reference in natural order.",
	userDocsLink: "https://evitadb.io/documentation/query/ordering/reference#reference-property"
};
var priceNatural = {
	shortDescription: "The constraint sorts returned entities by selected price for sale.",
	userDocsLink: "https://evitadb.io/documentation/query/ordering/price#price-natural"
};
var entityPrimaryKeyExact = {
	shortDescription: "The constraint sorts returned entities by ordering of the values specified in arguments.",
	userDocsLink: "https://evitadb.io/documentation/query/ordering/constant#exact-entity-primary-key-order"
};
var attributeNatural = {
	shortDescription: "The constraint sorts returned entities by natural ordering of the values in the specified attribute.",
	userDocsLink: "https://evitadb.io/documentation/query/ordering/comparable#attribute-natural"
};
var orderGroupBy = {
	shortDescription: "The container encapsulates order constraints that control the order of the facet groups in facet summary.",
	userDocsLink: "https://evitadb.io/documentation/query/basics#order-by"
};
var orderBy = {
	shortDescription: "The container encapsulates inner order constraints into one main constraint that is required by the query.",
	userDocsLink: "https://evitadb.io/documentation/query/basics#order-by"
};
var attributeSetInFilter = {
	shortDescription: "The constraint sorts returned entities by ordering of the values specified `attributeInSet` in filter sharing the same attribute name.",
	userDocsLink: "https://evitadb.io/documentation/query/ordering/constant#exact-entity-attribute-value-order-used-in-filter"
};
var attributeSetExact = {
	shortDescription: "The constraint sorts returned entities by ordering of the values specified in arguments matching the entity attribute of specified name.",
	userDocsLink: "https://evitadb.io/documentation/query/ordering/constant#exact-entity-attribute-value-order"
};
var entityGroupProperty = {
	shortDescription: "The constraint sorts returned references by applying ordering constraint on referenced entity group.",
	userDocsLink: "https://evitadb.io/documentation/query/ordering/reference#entity-group-property"
};
var entityPrimaryKeyNatural = {
	shortDescription: "The constraint sorts returned entities by primary key in specific order.",
	userDocsLink: "https://evitadb.io/documentation/query/ordering/comparable#primary-key-natural"
};
var random = {
	shortDescription: "The constraint sorts returned entities randomly.",
	userDocsLink: "https://evitadb.io/documentation/query/ordering/random#random"
};
var hierarchyOfReference = {
	shortDescription: "The constraint triggers computation of hierarchy statistics (how many matching children the hierarchy nodes have) of referenced hierarchical entities into response.",
	userDocsLink: "https://evitadb.io/documentation/query/requirements/hierarchy#hierarchy-of-reference"
};
var priceType = {
	shortDescription: "The constraint specifies which price type (with/without tax) will be used for handling filtering and sorting constraints.",
	userDocsLink: "https://evitadb.io/documentation/query/requirements/price#price-type"
};
var dataInLocales = {
	shortDescription: "The constraint triggers fetching of the localized attributes or associated data in different/additional locales than the locale specified in filtering constraints (if any at all).",
	userDocsLink: "https://evitadb.io/documentation/query/requirements/fetching#data-in-locales"
};
var hierarchyOfSelf = {
	shortDescription: "The constraint triggers computation of hierarchy statistics (how many matching children the hierarchy nodes have) of same hierarchical collection into response.",
	userDocsLink: "https://evitadb.io/documentation/query/requirements/hierarchy#hierarchy-of-self"
};
var fromNode = {
	shortDescription: "The constraint triggers computing the hierarchy subtree starting at pivot node.",
	userDocsLink: "https://evitadb.io/documentation/query/requirements/hierarchy#from-node"
};
var distance = {
	shortDescription: "The constraint limits the traversing in stop at container at specified distance (number of nodes in path).",
	userDocsLink: "https://evitadb.io/documentation/query/requirements/hierarchy#distance"
};
var stopAt = {
	shortDescription: "The constraint defines the traversal stop condition that limits the scope of the returned hierarchy tree.",
	userDocsLink: "https://evitadb.io/documentation/query/requirements/hierarchy#stop-at"
};
var facetSummaryOfReference = {
	shortDescription: "The constraint triggers computation of facet summary of all facet in searched scope into response with custom \\fetching\\ settings for specific reference.",
	userDocsLink: "https://evitadb.io/documentation/query/requirements/facet#facet-summary-of-reference"
};
var level = {
	shortDescription: "The constraint limits the traversing in stop at container at specified level from root.",
	userDocsLink: "https://evitadb.io/documentation/query/requirements/hierarchy#level"
};
var facetGroupsDisjunction = {
	shortDescription: "Sets relation of facets in the specified groups towards facets in different groups to [logical OR](https://en.wikipedia.org/wiki/Logical_disjunction) .",
	userDocsLink: "https://evitadb.io/documentation/query/requirements/facet#facet-groups-disjunction"
};
var children = {
	shortDescription: "The constraint triggers computing the hierarchy subtree starting at currently requested hierarchy node in filter by constraint.",
	userDocsLink: "https://evitadb.io/documentation/query/requirements/hierarchy#children"
};
var facetGroupsConjunction = {
	shortDescription: "Sets inter-facets relation within the specified groups to [logical AND](https://en.wikipedia.org/wiki/Logical_conjunction).",
	userDocsLink: "https://evitadb.io/documentation/query/requirements/facet#facet-groups-conjunction"
};
var fromRoot = {
	shortDescription: "The constraint triggers computing the hierarchy subtree starting at root level.",
	userDocsLink: "https://evitadb.io/documentation/query/requirements/hierarchy#from-root"
};
var queryTelemetry = {
	shortDescription: "The constraint triggers computation of query telemetry (explaining what operations were performed and how long they took) in extra results of the response.",
	userDocsLink: "https://evitadb.io/documentation/query/requirements/debug#query-telemetry"
};
var priceHistogram = {
	shortDescription: "The constraint triggers computation of the [histogram](https://en.wikipedia.org/wiki/Histogram) of price for sale into response.",
	userDocsLink: "https://evitadb.io/documentation/query/requirements/histogram#price-histogram"
};
var associatedDataContent = {
	shortDescription: "The constraint triggers fetching the entity associated data of specified names into the returned entities.",
	userDocsLink: "https://evitadb.io/documentation/query/requirements/fetching#associated-data-content"
};
var hierarchyContent = {
	shortDescription: "The constraint triggers fetching parent hierarchy entity parent chain and its bodies into returned main entities.",
	userDocsLink: "https://evitadb.io/documentation/query/requirements/fetching#hierarchy-content"
};
var statistics = {
	shortDescription: "The constraint triggers computing the count of children for each returned hierarchy node.",
	userDocsLink: "https://evitadb.io/documentation/query/requirements/hierarchy#statistics"
};
var facetGroupsNegation = {
	shortDescription: "[Negates](https://en.wikipedia.org/wiki/Negation) the meaning of selected facets in specified  + facet groups in the sense that their selection would return entities that don't have any of those facets.",
	userDocsLink: "https://evitadb.io/documentation/query/requirements/facet#facet-groups-negation"
};
var entityFetch = {
	shortDescription: "Returns richer entities instead of just entity references (empty container returns only entity body).",
	userDocsLink: "https://evitadb.io/documentation/query/requirements/fetching#entity-fetch"
};
var require$1 = {
	shortDescription: "The container encapsulates inner require constraints into one main constraint that is required by the query",
	userDocsLink: "https://evitadb.io/documentation/query/basics#require"
};
var entityGroupFetch = {
	shortDescription: "Returns richer group entities instead of just entity references (empty container returns only entity body).",
	userDocsLink: "https://evitadb.io/documentation/query/requirements/fetching#entity-group-fetch"
};
var node = {
	shortDescription: "The constraint allows to locate the pivot hierarchy node.",
	userDocsLink: "https://evitadb.io/documentation/query/requirements/hierarchy#node"
};
var attributeContent = {
	shortDescription: "The constraint triggers fetching the entity attributes into the returned entities.",
	userDocsLink: "https://evitadb.io/documentation/query/requirements/fetching#attribute-content"
};
var strip = {
	shortDescription: "The constraint specifies which strip (subset) of found entities will be returned.",
	userDocsLink: "https://evitadb.io/documentation/query/requirements/paging#strip"
};
var siblings = {
	shortDescription: "The constraint triggers computing the sibling axis for currently requested hierarchy node in filter by constraint or processed node by hierarchy parents axis.",
	userDocsLink: "https://evitadb.io/documentation/query/requirements/hierarchy#siblings"
};
var priceContent = {
	shortDescription: "The constraint triggers fetching the entity prices into the returned entities.",
	userDocsLink: "https://evitadb.io/documentation/query/requirements/fetching#price-content"
};
var page = {
	shortDescription: "The constraint specifies which page of found entities will be returned.",
	userDocsLink: "https://evitadb.io/documentation/query/requirements/paging#page"
};
var parents = {
	shortDescription: "The constraint triggers computing the hierarchy parent axis starting at currently requested hierarchy node in filter by constraint.",
	userDocsLink: "https://evitadb.io/documentation/query/requirements/hierarchy#parents"
};
var referenceContent = {
	shortDescription: "The constraint triggers fetching referenced entity bodies into returned main entities.",
	userDocsLink: "https://evitadb.io/documentation/query/requirements/fetching#reference-content"
};
var attributeHistogram = {
	shortDescription: "The constraint triggers computation of the [histogram](https://en.wikipedia.org/wiki/Histogram) of specified attributes into response.",
	userDocsLink: "https://evitadb.io/documentation/query/requirements/histogram#attribute-histogram"
};
var facetSummary = {
	shortDescription: "The constraint triggers computation of facet summary of all facet in searched scope into response with default \\fetching\\ settings for all referenced entities.",
	userDocsLink: "https://evitadb.io/documentation/query/requirements/facet#facet-summary"
};
var constraints = {
	collection: collection,
	filterGroupBy: filterGroupBy,
	attributeGreaterThanEquals: attributeGreaterThanEquals,
	filterBy: filterBy,
	priceBetween: priceBetween,
	entityHaving: entityHaving,
	priceInCurrency: priceInCurrency,
	userFilter: userFilter,
	attributeContains: attributeContains,
	excluding: excluding,
	priceValidIn: priceValidIn,
	attributeLessThan: attributeLessThan,
	priceInPriceLists: priceInPriceLists,
	entityLocaleEquals: entityLocaleEquals,
	attributeGreaterThan: attributeGreaterThan,
	facetHaving: facetHaving,
	or: or,
	attributeEquals: attributeEquals,
	hierarchyWithinRoot: hierarchyWithinRoot,
	and: and,
	attributeBetween: attributeBetween,
	referenceHaving: referenceHaving,
	not: not,
	attributeIs: attributeIs,
	attributeLessThanEquals: attributeLessThanEquals,
	having: having,
	attributeStartsWith: attributeStartsWith,
	hierarchyWithin: hierarchyWithin,
	directRelation: directRelation,
	attributeEndsWith: attributeEndsWith,
	excludingRoot: excludingRoot,
	attributeInSet: attributeInSet,
	entityPrimaryKeyInSet: entityPrimaryKeyInSet,
	attributeInRange: attributeInRange,
	entityProperty: entityProperty,
	entityPrimaryKeyInFilter: entityPrimaryKeyInFilter,
	referenceProperty: referenceProperty,
	priceNatural: priceNatural,
	entityPrimaryKeyExact: entityPrimaryKeyExact,
	attributeNatural: attributeNatural,
	orderGroupBy: orderGroupBy,
	orderBy: orderBy,
	attributeSetInFilter: attributeSetInFilter,
	attributeSetExact: attributeSetExact,
	entityGroupProperty: entityGroupProperty,
	entityPrimaryKeyNatural: entityPrimaryKeyNatural,
	random: random,
	hierarchyOfReference: hierarchyOfReference,
	priceType: priceType,
	dataInLocales: dataInLocales,
	hierarchyOfSelf: hierarchyOfSelf,
	fromNode: fromNode,
	distance: distance,
	stopAt: stopAt,
	facetSummaryOfReference: facetSummaryOfReference,
	level: level,
	facetGroupsDisjunction: facetGroupsDisjunction,
	children: children,
	facetGroupsConjunction: facetGroupsConjunction,
	fromRoot: fromRoot,
	queryTelemetry: queryTelemetry,
	priceHistogram: priceHistogram,
	associatedDataContent: associatedDataContent,
	hierarchyContent: hierarchyContent,
	statistics: statistics,
	facetGroupsNegation: facetGroupsNegation,
	entityFetch: entityFetch,
	require: require$1,
	entityGroupFetch: entityGroupFetch,
	node: node,
	attributeContent: attributeContent,
	strip: strip,
	siblings: siblings,
	priceContent: priceContent,
	page: page,
	parents: parents,
	referenceContent: referenceContent,
	attributeHistogram: attributeHistogram,
	facetSummary: facetSummary
};

// This file was generated by lezer-generator. You probably shouldn't edit it.
const parser = lr.LRParser.deserialize({
  version: 14,
  states: "'bOYQPOOO_QPO'#C_QOQPOOOdQPO'#CbOOQO,58y,58yOrQPO'#CcOwQPO,58|O!}QQO'#CdOOQO,58},58}OdQPO'#CtO#UQPO1G.hOOQO1G.h1G.hO#^QQO'#DPOOQO'#Co'#CoOOQO'#DO'#DOOrQPO'#CrOOQO'#C}'#C}OOQO,59O,59OO#lQPO,59OOOQO,59`,59`OOQO-E6r-E6rOOQO7+$S7+$SO#tQPO,59kO$SQQO,59oO$XQQO,59pO$^QQO,59qOOQO,59^,59^O!PQQO'#CsO$cQPO1G.jOOQO1G.j1G.jO$kQPO1G/VO%VQPO1G/ZO%[QPO1G/[O%aQPO1G/]O%fQPO1G/ZO%nQPO1G/[O%vQPO1G/]OOQO,59_,59_OOQO-E6q-E6qOOQO7+$U7+$UOOQO7+$u7+$uOOQO7+$v7+$vOOQO7+$w7+$wO&OQPO7+$uO&TQPO7+$vO&YQPO7+$wOOQO<<Ha<<HaOOQO<<Hb<<HbOOQO<<Hc<<Hc",
  stateData: "&n~OkOSPOS~OlPO~OTRO~OmTOnTOoTOpTO~OTVO~OSZO{XO~OX^OY^OZ^O[^O]^O^^O_^O`^Oa^Ob^Od^Oe^Ot[Oz_O~OSaO~P!POSeO{XO~O[iO]hObgOufO~OSmO{kO~O[qO]pOboOvnO~OurO~OusO~OutO~OSwO{kO~OSsiSwiSxiSyi{si{wi{xi{yi~OvxO~OvyO~OvzO~Ob{OvxO~O]|OvyO~O[}OvzO~Ov!OO~Ov!PO~Ov!QO~OzZba_`][^deYX]~",
  goto: "!}zPPP{PP!O!R!XPPPPPPPPPP!_PP!c!g!mPPPPPPPP!s!c!yPPP!y!y!yRQORSPQURRcXQWTRj_T^VkT`VkQlbRvlQYURdYQbVRukT]Vk",
  nodeNames: "⚠ Comment Request Query ArgsClosing ArgsOpening QueryBody RootConstraint ConstraintBody PositionalParameter NamedParameter String Int Float Boolean Date Time DateTime OffsetDateTime Range Uuid Enum Constraint",
  maxTerm: 43,
  nodeProps: [
    ["openedBy", 4,"ArgsOpening"],
    ["closedBy", 5,"ArgsClosing"]
  ],
  skippedNodes: [0,1],
  repeatNodeCount: 2,
  tokenData: "#@U~RuXY#fYZ#f]^#fpq#frs#wwx%exy'qyz'v|}'{}!O(S!P!Q(s!Q![)b!a!b2k!b!c2p!c!}3U!}#O3g#P#Q3l#T#U3q#U#V1X#V#WJt#W#X!#S#X#Y!*[#Y#Z!5k#Z#[1X#[#]!A_#]#`1X#`#a!I^#a#b1X#b#c!Jx#c#d!Kt#d#e!NP#e#f#)w#f#g#,p#g#h#6{#h#i#=w#i#j#>m#j#o1X~#kSk~XY#fYZ#f]^#fpq#f~#zVOr#wrs$as#O#w#O#P$f#P;'S#w;'S;=`%_<%lO#w~$fOZ~~$iRO;'S#w;'S;=`$r;=`O#w~$uWOr#wrs$as#O#w#O#P$f#P;'S#w;'S;=`%_;=`<%l#w<%lO#w~%bP;=`<%l#w~%hWOr%esw%ewx&Qx#O%e#O#P&o#P;'S%e;'S;=`'k<%lO%e~&VWZ~Or%esw%ewx&Qx#O%e#O#P&o#P;'S%e;'S;=`'k<%lO%e~&rRO;'S%e;'S;=`&{;=`O%e~'OXOr%esw%ewx&Qx#O%e#O#P&o#P;'S%e;'S;=`'k;=`<%l%e<%lO%e~'nP;=`<%l%e~'vOT~~'{OS~R(SOuQ{P~(VP!Q![(Y~(_Q[~!O!P(e!Q![(Y~(hP!Q![(k~(pP]~!Q![(k~(vP!P!Q(y~)OSP~OY(yZ;'S(y;'S;=`)[<%lO(y~)_P;=`<%l(y~)gS[~}!O)s!O!P(e!Q![+j#T#o1XQ)vQ!Q![)|#T#o)|Q*PR}!O*Y!Q![)|#T#o)|Q*]Q!Q![*c#T#o*cQ*fR}!O*o!Q![*c#T#o*cQ*rQ!Q![*x#T#o*xQ*{R}!O+U!Q![*x#T#o*xQ+XQ!Q![+_#T#o+_Q+dQdQ!Q![+_#T#o+_~+oT[~}!O)s!O!P(e!Q![,O![!]1e#T#o1X~,TS[~}!O)s!O!P(e!Q![,a#T#o1X~,fS[~}!O,r!O!P(e!Q![0v#T#o1X~,uQ!Q![,{#T#o)|~-OR}!O*Y!Q![-X#T#o)|~-[R}!O-e!Q![)|#T#o)|~-hQ!Q![-n#T#o*c~-qR}!O*o!Q![-z#T#o*c~.PS_~}!O*o!Q![*c!v!w.]#T#o*c~.`P!Q![.c~.fP!Q![.i~.lP![!].o~.rP!Q![.u~.xP!Q![.{~/OP![!]/R~/UP!Q![/X~/[P!Q![/_~/dRa~{|/m}!O/m!O!P0b~/pP!Q![/s~/vP!Q![/y~/|P![!]0P~0SP!Q![0V~0YP!Q![0]~0bOb~~0eP!Q![0h~0mRa~{|/m}!O/m!Q![0h~0{S[~}!O)s!O!P(e!Q![0v#T#o1XQ1[R}!O)s!Q![1X#T#o1X~1hP!Q![1k~1nP!Q![1q~1tP![!]1w~1zP!Q![1}~2QP!Q![2T~2YP`~!O!P2]~2`P!Q![2c~2hP`~!Q![2c~2pOX~~2sP#T#o2v~2{RY~!Q![2v!c!}2v#T#o2v~3ZQe~!c!}3U#R#S3a~3dP!c!}3U~3lOt~~3qOv~Q3tW}!O)s!Q![1X#T#b1X#b#c4^#c#g1X#g#h5O#h#i:Z#i#o1XQ4aT}!O)s!Q![1X#T#W1X#W#X4p#X#o1XQ4uRzQ}!O)s!Q![1X#T#o1XQ5RT}!O)s!Q![1X#T#g1X#g#h5b#h#o1XQ5eT}!O)s!Q![1X#T#c1X#c#d5t#d#o1XQ5wT}!O)s!Q![1X#T#V1X#V#W6W#W#o1XQ6ZT}!O)s!Q![1X#T#]1X#]#^6j#^#o1XQ6mS}!O)s!Q![1X#T#U6y#U#o1XQ6|T}!O)s!Q![1X#T#h1X#h#i7]#i#o1XQ7`T}!O)s!Q![1X#T#X1X#X#Y7o#Y#o1XQ7rT}!O)s!Q![1X#T#W1X#W#X8R#X#o1XQ8US}!O)s!Q![1X!f!g8b#T#o1XQ8eP#T#U8hQ8kP#h#i8nQ8qP#T#U8tQ8wP!e!f8zQ8}P#c#d9QQ9TP#b#c9WQ9ZP#h#i9^Q9aP#X#Y9dQ9gP#b#c9jQ9mP#h#i9pQ9uPzQ!c!d9xQ9{P#`#a:OQ:RP#`#a:UQ:ZOzQQ:^T}!O)s!Q![1X#T#h1X#h#i:m#i#o1XQ:pT}!O)s!Q![1X#T#f1X#f#g;P#g#o1XQ;ST}!O)s!Q![1X#T#]1X#]#^;c#^#o1XQ;fT}!O)s!Q![1X#T#U1X#U#V;u#V#o1XQ;xT}!O)s!Q![1X#T#i1X#i#j<X#j#o1XQ<[T}!O)s!Q![1X#T#h1X#h#i<k#i#o1XQ<nT}!O)s!Q![1X#T#X1X#X#Y<}#Y#o1XQ=Q[}!O)s!Q![1X!d!e=v!e!f>l!g!h?k!i!jBT!j!kDT!k!lEV!n!oGh!p!qGz!u!vHj#T#o1XQ=yP#X#Y=|Q>PP#h#i>SQ>VP#k#l>YQ>]P#X#Y>`Q>cP#X#Y>fQ>iP#b#c:UQ>oP#c#d>rQ>uP#b#c>xQ>{P#h#i?OQ?RQ#T#U?X#X#Y9dQ?[P#]#^?_Q?bP#b#c?eQ?hP#g#h:UQ?nQ#b#c?t#e#f@jQ?wP#W#X?zQ?}P#g#h@QQ@TP!y!z@WQ@ZP#]#^@^Q@aP#h#i@dQ@gP#[#]:UQ@mP#i#j@pQ@sP#T#U@vQ@yP#`#a@|QAPP#g#hASQAXQzQ!h!iA_!v!wAwQAbP#T#UAeQAhP#`#aAkQAnP#g#hAqQAtP#X#Y:UQAzP#f#gA}QBQP#i#jAqQBWP#f#gBZQB^P#X#YBaQBdP#T#UBgQBjP#h#iBmQBpP#X#YBsQBvP#f#gByQB|P!v!wCPQCSP#[#]CVQCYP#T#UC]QC`P#b#cCcQChPzQ!g!hCkQCnP#e#fCqQCtP#i#jCwQCzP#T#UC}QDQP#`#a?eQDWP#]#^DZQD^P#g#hDaQDdP#h#iDgQDjP#c#dDmQDpP#Z#[DsQDvP#f#gDyQD|P#T#UEPQESP#a#b:UQEYQ#b#cE`#g#hFsQEcQ!t!uEi!u!vFgQElP#T#UEoQErP#b#cEuQExP#Z#[E{QFOP#X#YFRQFWPzQ!p!qFZQF^P#c#dFaQFdP#k#l:UQFjP#X#YFmQFpP#h#i:UQFxPzQ!p!qF{QGOQ#c#dGU#i#j9xQGXP#h#iG[QG_P!p!qGbQGeP#i#j9xQGkP#X#YGnQGqP#g#hGtQGwP#g#hByQG}P#T#UHQQHTP#h#iHWQHZP#i#jH^QHaP#f#gHdQHgP#T#U:OQHmQ#X#YHs#h#iJbQHvP#h#iHyQH|Q!g!hIS!k!lIfQIVP#l#mIYQI]P#T#UI`QIcP#V#WFmQIiP#b#cIlQIoP!h!iIrQIuP#]#^IxQI{P#`#aJOQJRP#h#iJUQJXP#X#YJ[QJ_P#f#g:UQJeP#T#UJhQJkP#f#gJnQJqP#h#i?zRJwV}!O)s!Q![1X#T#[1X#[#]K^#]#c1X#c#dMn#d#o1XQKaT}!O)s!Q![1X#T#]1X#]#^Kp#^#o1XQKsT}!O)s!Q![1X#T#`1X#`#aLS#a#o1XQLVT}!O)s!Q![1X#T#W1X#W#XLf#X#o1XQLiT}!O)s!Q![1X#T#f1X#f#gLx#g#o1XQL{T}!O)s!Q![1X#T#X1X#X#YM[#Y#o1XQM_T}!O)s!Q![1X#T#b1X#b#c4p#c#o1XRMqT}!O)s!Q![1X#T#`1X#`#aNQ#a#o1XRNTT}!O)s!Q![1X#T#`1X#`#aNd#a#o1XRNgT}!O)s!Q![1X#T#X1X#X#YNv#Y#o1XRNyT}!O)s!Q![1X#T#V1X#V#W! Y#W#o1XR! ]T}!O)s!Q![1X#T#h1X#h#i! l#i#o1XR! oT}!O)s!Q![1X#T#]1X#]#^!!O#^#o1XR!!RT}!O)s!Q![1X#T#c1X#c#d!!b#d#o1XR!!eT}!O)s!Q![1X#T#b1X#b#c!!t#c#o1XR!!yRmP}!O)s!Q![1X#T#o1XQ!#VU}!O)s!Q![1X#T#U!#i#U#]1X#]#^!%m#^#o1XQ!#lT}!O)s!Q![1X#T#h1X#h#i!#{#i#o1XQ!$OS}!O)s!Q![1X#T#U!$[#U#o1XQ!$_S}!O)s!Q![1X!k!l!$k#T#o1XQ!$nP#b#c!$qQ!$tP!n!o!$wQ!$zP#c#d!$}Q!%QP#V#W!%TQ!%WP#T#U!%ZQ!%^P#`#a!%aQ!%dP#X#Y!%gQ!%jP#g#h9pQ!%pU}!O)s!Q![1X#T#f1X#f#g!&S#g#h!(a#h#o1XQ!&VT}!O)s!Q![1X#T#X1X#X#Y!&f#Y#o1XQ!&iT}!O)s!Q![1X#T#V1X#V#W!&x#W#o1XQ!&{T}!O)s!Q![1X#T#h1X#h#i!'[#i#o1XQ!'_S}!O)s!Q![1X!t!u!'k#T#o1XQ!'nP#X#Y!'qQ!'tP#`#a!'wQ!'zP#T#U!'}Q!(QP#h#i!(TQ!(WP#]#^!(ZQ!(^P#c#d>fQ!(dT}!O)s!Q![1X#T#h1X#h#i!(s#i#o1XQ!(vS}!O)s!Q![1X#T#U!)S#U#o1XQ!)VT}!O)s!Q![1X#T#b1X#b#c!)f#c#o1XQ!)iT}!O)s!Q![1X#T#V1X#V#W!)x#W#o1XQ!){T}!O)s!Q![1X#T#X1X#X#Y4p#Y#o1XQ!*_V}!O)s!Q![1X#T#b1X#b#c!*t#c#l1X#l#m!2Y#m#o1XQ!*wT}!O)s!Q![1X#T#h1X#h#i!+W#i#o1XQ!+ZT}!O)s!Q![1X#T#]1X#]#^!+j#^#o1XQ!+mT}!O)s!Q![1X#T#h1X#h#i!+|#i#o1XQ!,PT}!O)s!Q![1X#T#m1X#m#n!,`#n#o1XQ!,cW}!O)s!Q![1X!h!i!,{!i!j!-_!j!k!.|!n!o!/l!r!s!0b#T#o1XQ!-OP#X#Y!-RQ!-UP#h#i!-XQ!-[P#V#W@dQ!-bP#f#g!-eQ!-hP#c#d!-kQ!-nP#i#j!-qQ!-tP#d#e!-wQ!-zQ!h!i!,{!r!s!.QQ!.TP#f#g!.WQ!.ZP#c#d!.^Q!.aP#d#e!.dQ!.gP#X#Y!.jQ!.mP#f#g!.pQ!.sP#h#i!.vQ!.yP#m#n:UQ!/PP#T#U!/SQ!/VP#j#k!/YQ!/]P#]#^!/`Q!/cP#b#c!/fQ!/iP#Z#[:UQ!/oP#c#d!/rQ!/uP#V#W!/xQ!/{P#T#U!0OQ!0RP#`#a!0UQ!0XP#X#Y!0[Q!0_P!g!hCkQ!0eP#f#g!0hQ!0kQ#]#^!0q#c#d!.^Q!0tP#a#b!0wQ!0zP#T#U!0}Q!1QP#f#g!1TQ!1WP#m#n!1ZQ!1^P!m!n!1aQ!1dP#X#Y!1gQ!1jP#m#n!1mQ!1pR!g!hIS!k!l!1y!p!qGzQ!1|P#b#c!2PQ!2SQ!h!iIr!u!vFgQ!2]T}!O)s!Q![1X#T#V1X#V#W!2l#W#o1XQ!2oT}!O)s!Q![1X#T#`1X#`#a!3O#a#o1XQ!3RT}!O)s!Q![1X#T#i1X#i#j!3b#j#o1XQ!3eT}!O)s!Q![1X#T#W1X#W#X!3t#X#o1XQ!3wT}!O)s!Q![1X#T#]1X#]#^!4W#^#o1XQ!4ZT}!O)s!Q![1X#T#b1X#b#c!4j#c#o1XQ!4mT}!O)s!Q![1X#T#Z1X#Z#[!4|#[#o1XQ!5RSzQ}!O)s!Q![1X!t!u!5_#T#o1XQ!5bP#c#d!5eQ!5hP#c#dFm~!5nW}!O)s!Q![1X#T#U!6W#U#]1X#]#^!=a#^#f1X#f#g!@Y#g#o1X~!6ZV}!O)s!Q![1X#T#V1X#V#W!6p#W#`1X#`#a!<]#a#o1XQ!6sT}!O)s!Q![1X#T#X1X#X#Y!7S#Y#o1XQ!7VT}!O)s!Q![1X#T#h1X#h#i!7f#i#o1XQ!7iU}!O)s!Q![1X!i!j!7{!j!k!.|!u!v!:V#T#o1XQ!8OP#f#g!8RQ!8UP#c#d!8XQ!8[P#i#j!8_Q!8bP#d#e!8eQ!8hP#g#h!8kQ!8nR!e!f!8w!f!g!9m!p!q!9yQ!8zP#c#d!8}Q!9QP#b#c!9TQ!9WP#^#_!9ZQ!9^P#i#j!9aQ!9dP#b#c!9gQ!9jP#V#W!'}Q!9pP#]#^!9sQ!9vP#g#h!9TQ!9|P#X#Y!:PQ!:SP#Z#[!'wQ!:YP#i#j!:]Q!:`P#a#b!:cQ!:fP#a#b!:iQ!:lP#T#U!:oQ!:rP#f#g!:uQ!:xP#m#n!:{Q!;QPzQ!q!r!;TQ!;WP#Y#Z!;ZQ!;^P!t!u!;aQ!;dP#X#Y!;gQ!;jP#Y#Z!;mQ!;pP#X#Y!;sQ!;vP#f#g!;yQ!;|P#X#Y!<PQ!<SP#b#c!<VQ!<YP#V#WAq~!<`T}!O)s!Q![1X#T#g1X#g#h!<o#h#o1X~!<rT}!O)s!Q![1X#T#X1X#X#Y!=R#Y#o1X~!=WR^~}!O)s!Q![1X#T#o1X~!=dT}!O)s!Q![1X#T#`1X#`#a!=s#a#o1X~!=vT}!O)s!Q![1X#T#h1X#h#i!>V#i#o1X~!>YT}!O)s!Q![1X#T#X1X#X#Y!>i#Y#o1X~!>lT}!O)s!Q![1X#T#f1X#f#g!>{#g#o1X~!?OT}!O)s!Q![1X!d!e!?_!i!j!?j#T#o1X~!?bP#m#n!?e~!?jOn~Q!?mP#f#g!?pQ!?sP#c#d!?vQ!?yP#i#j!?|Q!@PP#d#e!@SQ!@VP!d!e!.vQ!@]T}!O)s!Q![1X#T#c1X#c#d!@l#d#o1XQ!@oT}!O)s!Q![1X#T#a1X#a#b!AO#b#o1XQ!ARS}!O)s!Q![1X!t!u!5_#T#o1XQ!AbU}!O)s!Q![1X#T#U!At#U#]1X#]#^!C`#^#o1XQ!AwT}!O)s!Q![1X#T#j1X#j#k!BW#k#o1XQ!BZT}!O)s!Q![1X#T#]1X#]#^!Bj#^#o1XQ!BmT}!O)s!Q![1X#T#b1X#b#c!B|#c#o1XQ!CPT}!O)s!Q![1X#T#Z1X#Z#[4p#[#o1XQ!CcT}!O)s!Q![1X#T#X1X#X#Y!Cr#Y#o1XQ!CuT}!O)s!Q![1X#T#f1X#f#g!DU#g#o1XQ!DXS}!O)s!Q![1X#T#U!De#U#o1XQ!DhT}!O)s!Q![1X#T#f1X#f#g!Dw#g#o1XQ!DzT}!O)s!Q![1X#T#V1X#V#W!EZ#W#o1XQ!E^T}!O)s!Q![1X#T#[1X#[#]!Em#]#o1XQ!EpT}!O)s!Q![1X#T#m1X#m#n!FP#n#o1XQ!FSU}!O)s!Q![1X!e!f!Ff!q!r!GU!y!z!Gw#T#o1XQ!FiP#c#d!FlQ!FoP#b#c!FrQ!FuP#h#i!FxQ!F{P#X#Y!GOQ!GRP#b#cFmQ!GXP#Y#Z!G[Q!G_Q!t!u!;a!u!v!GeQ!GhP#X#Y!GkQ!GnP#`#a!GqQ!GtP#Y#Z:UQ!GzP#]#^!G}Q!HQP#h#i!HTQ!HWP#[#]!HZQ!H^P#]#^!HaQ!HdP#b#c!HgQ!HlQzQ!t!u!Hr!u!v!GeQ!HuP#c#d!HxQ!H{P#c#d!IOQ!IRP#h#i!IUQ!IZPzQ!u!v!GeQ!IaT}!O)s!Q![1X#T#X1X#X#Y!Ip#Y#o1XQ!IsT}!O)s!Q![1X#T#j1X#j#k!JS#k#o1XQ!JVT}!O)s!Q![1X#T#X1X#X#Y!Jf#Y#o1XQ!JiT}!O)s!Q![1X#T#`1X#`#a4p#a#o1XQ!J{T}!O)s!Q![1X#T#c1X#c#d!K[#d#o1XQ!K_V}!O)s!Q![1X#T#W1X#W#X!)x#X#h1X#h#i4p#i#o1XR!KwT}!O)s!Q![1X#T#f1X#f#g!LW#g#o1XR!L]TzQ}!O)s!Q![1X#T#W1X#W#X!Ll#X#o1XR!LoT}!O)s!Q![1X#T#X1X#X#Y!MO#Y#o1XR!MRT}!O)s!Q![1X#T#f1X#f#g!Mb#g#o1XR!MeT}!O)s!Q![1X!d!e!Mt!i!j!?j#T#o1XP!MwP#m#n!MzP!NPOoPQ!NSU}!O)s!Q![1X#T#U!Nf#U#f1X#f#g#!j#g#o1XQ!NiV}!O)s!Q![1X#T#Z1X#Z#[!)x#[#f1X#f#g# O#g#o1XQ# RT}!O)s!Q![1X#T#X1X#X#Y# b#Y#o1XQ# eT}!O)s!Q![1X#T#b1X#b#c# t#c#o1XQ# wT}!O)s!Q![1X#T#h1X#h#i#!W#i#o1XQ#!ZT}!O)s!Q![1X#T#g1X#g#h4p#h#o1XQ#!mT}!O)s!Q![1X#T#]1X#]#^#!|#^#o1XQ##PT}!O)s!Q![1X#T#V1X#V#W##`#W#o1XQ##cT}!O)s!Q![1X#T#X1X#X#Y##r#Y#o1XQ##uY}!O)s!Q![1X!d!e=v!e!f#$e!j!kDT!k!l#&n!p!qGz!v!w#(u!x!y#)R#T#o1XQ#$hP#c#d#$kQ#$nP#b#c#$qQ#$tP#h#i#$wQ#$zP#X#Y#$}Q#%QP#b#c#%TQ#%WP#h#i#%ZQ#%`QzQ!c!d9x!t!u#%fQ#%iP#X#Y#%lQ#%oP#g#h#%rQ#%uP#d#e#%xQ#%{P#X#Y#&OQ#&RP#V#W#&UQ#&XP#h#i#&[Q#&_P#]#^#&bQ#&eP#b#c#&hQ#&kP#Z#[IlQ#&qP#b#c#&tQ#&wQ!e!f#&}!r!s#'sQ#'QP#i#j#'TQ#'WP#f#g#'ZQ#'^P#f#g#'aQ#'dP#X#Y#'gQ#'jP#b#c#'mQ#'pP#V#W!.vQ#'vP#f#g#'yQ#'|P#]#^#(PQ#(SP#V#W#(VQ#(YP#X#Y#(]Q#(`P!n!o#(cQ#(fP#]#^#(iQ#(lP#g#h#(oQ#(rP#h#i?eQ#(xP#m#n#({Q#)OP#d#eAqQ#)UP#T#U#)XQ#)[P#`#a#)_Q#)bP#]#^#)eQ#)hP#W#X#)kQ#)nP!k!l#)qQ#)tP#b#cFRR#)zT}!O)s!Q![1X#T#i1X#i#j#*Z#j#o1XR#*^T}!O)s!Q![1X#T#X1X#X#Y#*m#Y#o1XR#*pT}!O)s!Q![1X#T#f1X#f#g#+P#g#o1XR#+ST}!O)s!Q![1X#T#m1X#m#n#+c#n#o1XR#+hSlP}!O)s!Q![1X!v!w#+t#T#o1XQ#+wP#X#Y#+zQ#+}P#`#a#,QQ#,TP#X#Y#,WQ#,ZP#a#b#,^Q#,aP#X#Y#,dQ#,gP#h#i#,jQ#,mP#f#g!.vR#,sU}!O)s!Q![1X#T#U#-V#U#X1X#X#Y#.q#Y#o1XQ#-YT}!O)s!Q![1X#T#b1X#b#c#-i#c#o1XQ#-lT}!O)s!Q![1X#T#W1X#W#X#-{#X#o1XQ#.OT}!O)s!Q![1X#T#c1X#c#d#._#d#o1XQ#.bT}!O)s!Q![1X#T#a1X#a#b4p#b#o1XR#.tV}!O)s!Q![1X#T#Y1X#Y#Z#/Z#Z#e1X#e#f#5R#f#o1XQ#/^T}!O)s!Q![1X#T#X1X#X#Y#/m#Y#o1XQ#/pT}!O)s!Q![1X#T#f1X#f#g#0P#g#o1XQ#0ST}!O)s!Q![1X#T#X1X#X#Y#0c#Y#o1XQ#0fT}!O)s!Q![1X#T#b1X#b#c#0u#c#o1XQ#0xT}!O)s!Q![1X#T#V1X#V#W#1X#W#o1XQ#1[T}!O)s!Q![1X#T#X1X#X#Y#1k#Y#o1XQ#1nU}!O)s!Q![1X!e!f#2Q!j!k!.|!r!s!.Q#T#o1XQ#2TP#c#d#2WQ#2ZP#b#c#2^Q#2aP#h#i#2dQ#2gP#X#Y#2jQ#2mP#b#c#2pQ#2sP#h#i#2vQ#2{QzQ!c!d#3R!y!z#3gQ#3UP#`#a#3XQ#3[P#`#a#3_Q#3dPzQ!y!z#3gQ#3jP#]#^#3mQ#3pP#h#i#3sQ#3vP#[#]#3yQ#3|P!c!d#4PQ#4SP#h#i#4VQ#4YP#h#i#4]Q#4`P#f#g#4cQ#4fP#]#^#4iQ#4lP#U#V#4oQ#4rP#i#j#4uQ#4xP#h#i#4{Q#5OP#X#Y?eR#5UT}!O)s!Q![1X#T#i1X#i#j#5e#j#o1XR#5hT}!O)s!Q![1X#T#]1X#]#^#5w#^#o1XR#5zT}!O)s!Q![1X#T#f1X#f#g#6Z#g#o1XR#6^T}!O)s!Q![1X#T#X1X#X#Y#6m#Y#o1XR#6rRpP}!O)s!Q![1X#T#o1XQ#7OV}!O)s!Q![1X#T#]1X#]#^#7e#^#h1X#h#i#9c#i#o1XQ#7hT}!O)s!Q![1X#T#U1X#U#V#7w#V#o1XQ#7zT}!O)s!Q![1X#T#`1X#`#a#8Z#a#o1XQ#8^T}!O)s!Q![1X#T#]1X#]#^#8m#^#o1XQ#8pT}!O)s!Q![1X#T#b1X#b#c#9P#c#o1XQ#9ST}!O)s!Q![1X#T#Z1X#Z#[#!W#[#o1XQ#9fW}!O)s!Q![1X#T#U#:O#U#c1X#c#d#<`#d#f1X#f#g#=R#g#o1XQ#:RT}!O)s!Q![1X#T#h1X#h#i#:b#i#o1XQ#:eT}!O)s!Q![1X#T#]1X#]#^#:t#^#o1XQ#:wT}!O)s!Q![1X#T#g1X#g#h#;W#h#o1XQ#;ZT}!O)s!Q![1X#T#h1X#h#i#;j#i#o1XQ#;mT}!O)s!Q![1X#T#]1X#]#^#;|#^#o1XQ#<PT}!O)s!Q![1X#T#V1X#V#W#!W#W#o1XQ#<cT}!O)s!Q![1X#T#d1X#d#e#<r#e#o1XQ#<uS}!O)s!Q![1X!c!dFm#T#o1XQ#=UT}!O)s!Q![1X#T#]1X#]#^#=e#^#o1XQ#=hT}!O)s!Q![1X#T#d1X#d#e4p#e#o1X~#=zT}!O)s!Q![1X#T#f1X#f#g#>Z#g#o1X~#>^T}!O)s!Q![1X#T#i1X#i#j!<o#j#o1XQ#>pT}!O)s!Q![1X#T#g1X#g#h#?P#h#o1XQ#?ST}!O)s!Q![1X#T#X1X#X#Y#?c#Y#o1XQ#?fT}!O)s!Q![1X#T#f1X#f#g#?u#g#o1XQ#?xS}!O)s!Q![1X!h!iIr#T#o1X",
  tokenizers: [0, 1],
  topRules: {"Request":[0,2]},
  tokenPrec: 245
});

const evitaQLLanguage = language.LRLanguage.define({
    parser: parser.configure({
        props: [
            language.indentNodeProp.add({
                "QueryBody ConstraintBody": language.delimitedIndent({ closing: ')', align: true }),
                Range: language.delimitedIndent({ closing: ']', align: true })
            }),
            language.foldNodeProp.add({
                'QueryBody ConstraintBody': language.foldInside
            }),
            highlight.styleTags({
                String: highlight.tags.string,
                Int: highlight.tags.integer,
                Float: highlight.tags.float,
                Boolean: highlight.tags.bool,
                Date: highlight.tags.literal,
                Time: highlight.tags.literal,
                DateTime: highlight.tags.literal,
                OffsetDateTime: highlight.tags.literal,
                Uuid: highlight.tags.literal,
                Enum: highlight.tags.constant(highlight.tags.variableName),
                Query: highlight.tags.function(highlight.tags.variableName),
                RootConstraint: highlight.tags.function(highlight.tags.variableName),
                Constraint: highlight.tags.function(highlight.tags.variableName),
                Comment: highlight.tags.lineComment,
                ',': highlight.tags.separator,
                '( )': highlight.tags.paren,
                '[ ]': highlight.tags.squareBracket
            })
        ]
    }),
    languageData: {
        commentTokens: { line: '//' }
    }
});

const evitaQLCompletion = evitaQLLanguage.data.of({
    // autocomplete: getEvitaQLCompletions
    autocomplete: autocomplete.completeFromList([
        createCompletion('query', 'Query is the root construct for querying data.'),
        ...Object.keys(constraints).map(it => createCompletion(it))
    ])
});
function createCompletion(label, info) {
    const constraintDefinition = constraints[label];
    return {
        label,
        detail: '(...)',
        type: 'function',
        info: info ? info : constraintDefinition['shortDescription'] + "\n\n[Check detailed documentation](" + constraintDefinition['userDocsLink'] + ")",
        apply: (view, completion, from, to) => {
            view.dispatch({
                changes: { from, to, insert: label + '()' },
                selection: { anchor: from + label.length + 1 }
            });
        }
    };
}

const evitaQLLinter = lint.linter(view => {
    const diagnostics = [];
    language.syntaxTree(view.state).cursor().iterate(node => {
        if (node.type.isError) {
            diagnostics.push({
                from: node.from,
                to: node.to,
                severity: 'error',
                message: 'Syntax error.'
            });
        }
    });
    return diagnostics;
});

function evitaQL() {
    return new language.LanguageSupport(evitaQLLanguage, [evitaQLCompletion, evitaQLLinter]);
}

exports.evitaQL = evitaQL;
exports.evitaQLCompletion = evitaQLCompletion;
exports.evitaQLLanguage = evitaQLLanguage;
exports.evitaQLLinter = evitaQLLinter;
